import * as THREE from 'three';
import { GLTFLoader } from 'three/addons/loaders/GLTFLoader.js';
import { OrbitControls } from 'three/addons/controls/OrbitControls.js';
import { gsap } from 'https://cdn.skypack.dev/gsap';

export function initViewer(containerId, modelPath, modelFile) {
    const container = document.getElementById(containerId);

    // Renderer
    const renderer = new THREE.WebGLRenderer({ alpha: true });
    renderer.setSize(1000, 500);
    renderer.shadowMap.enabled = true;
    container.appendChild(renderer.domElement);

    // Scene
    const scene = new THREE.Scene();
    scene.background = new THREE.Color(0xffffff); // Set white background

    // Camera
    const camera = new THREE.PerspectiveCamera(5, 1000 / 500, 0.1, 200);
    camera.position.set(0, -5, 10); // Positioned directly in front of the object
    camera.lookAt(0, 10, 0); // Adjusted to look slightly upward at the object
    // Adjust Model Position (if needed)
    //bee.position.set(0, 0, 0); // Ensure the model's position is centered in the scene

    // Orbit Controls
    const controls = new OrbitControls(camera, renderer.domElement);
    controls.enableDamping = true;

    // Lighting
    const ambientLight = new THREE.AmbientLight(0xffffff, 1.3);
    scene.add(ambientLight);

    const pointLight = new THREE.PointLight(0xffffff, 1);
    pointLight.position.set(10, 10, 10);
    scene.add(pointLight);

    const hemisphereLight = new THREE.HemisphereLight(0xffffff, 0x444444, 1);
    scene.add(hemisphereLight);

    // Load 3D Model
    const loader = new GLTFLoader().setPath(modelPath);
    let bee, mixer;

    loader.load(
        modelFile,
        (gltf) => {
            bee = gltf.scene;
            scene.add(bee);

            // Setup animation mixer for wings flapping
            if (gltf.animations && gltf.animations.length > 0) {
                mixer = new THREE.AnimationMixer(bee);
                gltf.animations.forEach((clip) => mixer.clipAction(clip).play());
            }

            bee.traverse((node) => {
                if (node.isMesh) {
                    node.material.needsUpdate = true; // Retain original textures
                    node.castShadow = true; // Enable shadows
                    node.receiveShadow = true;
                }
            });

            // Set initial position and rotation
            bee.position.set(-0.4, -0.4, 0);
            bee.rotation.set(0, 0, 0);

            // Add subtle dynamic movement
            subtleMovement(bee);

            // Setup button handlers
            setupButtonHandlers(bee);
        },
        (xhr) => console.log(`Loading ${containerId}: ${(xhr.loaded / xhr.total) * 100}%`),
        (error) => console.error(`Error loading ${containerId}:`, error)
    );

    // Subtle movement
    function subtleMovement(bee) {
        if (!bee) return;

        // Slight up-and-down motion
        gsap.to(bee.position, {
            y: bee.position.y + 0.2,
            duration: 1.5,
            yoyo: true,
            repeat: -1,
            ease: 'sine.inOut',
        });

        // Slight rotation to simulate hovering
        gsap.to(bee.rotation, {
            z: bee.rotation.z + 0.05,
            duration: 1,
            yoyo: true,
            repeat: -1,
            ease: 'sine.inOut',
        });
    }

    // Button handlers
    function setupButtonHandlers(bee) {
        const rotateAndReturn = (direction) => {
            if (!bee) return;
    
            const targetRotation = direction === 'LEFT' ? -Math.PI / 3 : Math.PI / 3; // 60 degrees
            const originalRotation = bee.rotation.y; // Store original rotation
    
            // Animate to the target rotation, hold briefly, then return
            gsap.to(bee.rotation, {
                y: targetRotation,
                duration: 1, // Slow rotation
                onComplete: () => {
                    // Hold for a moment
                    gsap.to(bee.rotation, {
                        y: originalRotation,
                        duration: 1, // Return slowly
                        delay: 2, // Hold for half a second
                    });
                },
            });
        };
    
        document.querySelector('button[onclick*="LEFT"]').addEventListener('click', () => {
            rotateAndReturn('LEFT');
        });
    
        document.querySelector('button[onclick*="RIGHT"]').addEventListener('click', () => {
            rotateAndReturn('RIGHT');
        });
    
        document.querySelector('button[onclick*="BLINK"]').addEventListener('click', () => {
            if (bee) {
                // Rotate the Bee around its Z-axis for a blink effect
                gsap.to(bee.rotation, { z: bee.rotation.z + Math.PI * 2, duration: 0.5 });
            }
        });
    }
    

    // Resize handling
    window.addEventListener('resize', () => {
        renderer.setSize(1000, 500);
        camera.aspect = 1000 / 500;
        camera.updateProjectionMatrix();
    });

    // Animation Loop
    function animate() {
        requestAnimationFrame(animate);

        // Update mixer if animations are active
        if (mixer) mixer.update(0.01);

        controls.update();
        renderer.render(scene, camera);
    }

    animate();
}
